package com.qmth.teachcloud.cas.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.teachcloud.exchange.common.bean.params.OpenParams;
import com.qmth.teachcloud.exchange.common.bean.params.RedirectParams;
import com.qmth.teachcloud.exchange.common.contant.SystemConstant;
import com.qmth.teachcloud.exchange.common.entity.BasicSchool;
import com.qmth.teachcloud.exchange.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.exchange.common.service.AuthInfoService;
import com.qmth.teachcloud.exchange.common.service.CommonCacheService;
import com.qmth.teachcloud.exchange.common.util.HttpUtil;
import com.qmth.teachcloud.exchange.common.util.JacksonUtil;
import com.qmth.teachcloud.exchange.common.util.Result;
import com.qmth.teachcloud.exchange.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * @Description: 西交大cas工具类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2023/6/12
 */
@Component
public class XjtuCasUtil {
    private static final Logger log = LoggerFactory.getLogger(XjtuCasUtil.class);
    private String teachcloudLoginUrl = null, logoutUrl = null, returnUrl = null;
    private static final String LOGIN_BEFORE_XJU_LOGIC_API = "/api/admin/print/open/login_before_xju_logic";//西交大登录之前逻辑
    private static final String ACCESS_TOKEN_URL = "http://org.xjtu.edu.cn/openplatform/oauth/getAccessToken";
    private static final String USER_INFO_URL = "http://org.xjtu.edu.cn/openplatform/oauth/open/getUserInfo";
    private static final String LOGOUT_URL = "http://org.xjtu.edu.cn/openplatform/oauth/logout";

    @Resource
    AuthInfoService authInfoService;

    @Resource
    CommonCacheService commonCacheService;

    //    @Value("${cas.config.logoutUrl}")
//    String logoutUrl;
//
//    @Value("${cas.config.returnUrl}")
//    String returnUrl;
//
//    @Value("${cas.config.teachcloudPrintLoginUrl}")
//    String teachcloudLoginUrl;

    /**
     * 西交大登录逻辑
     *
     * @param schoolCode
     * @param code
     * @param state
     * @param userType
     * @param employeeNo
     * @param returnUrl
     * @return
     * @throws IOException
     */
    public RedirectParams authentication(String schoolCode,
                                         String code,
                                         String state,
                                         String userType,
                                         String employeeNo,
                                         String returnUrl) throws IOException {
        if ((Objects.isNull(code) || Objects.equals(code, ""))
                || (Objects.isNull(employeeNo) || Objects.equals(employeeNo, ""))) {
            throw ExceptionResultEnum.ERROR.exception("请先通过学校地址登录");
        }
        authInfoService.appHasExpired(schoolCode);

        //2022-12-06加入登录之前逻辑
        if (Objects.isNull(teachcloudLoginUrl) || Objects.equals(teachcloudLoginUrl, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("知学登录跳转地址不存在");
        }
        int index = teachcloudLoginUrl.lastIndexOf(SystemConstant.PATH_SUBSTR);
        String hostUrl = teachcloudLoginUrl.substring(0, index);
        if (hostUrl.contains(SystemConstant.PATH_MATCH)) {
            hostUrl = hostUrl.replace(SystemConstant.PATH_MATCH, schoolCode);
        }

        BasicSchool basicSchool = commonCacheService.schoolCache(schoolCode);
        Optional.ofNullable(basicSchool).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("学校信息不存在"));

        Long timestamp = System.currentTimeMillis();
        String signature = SignatureEntity.build(SignatureType.SECRET, SystemConstant.METHOD, LOGIN_BEFORE_XJU_LOGIC_API, timestamp, basicSchool.getAccessKey(), basicSchool.getAccessSecret());

        String callResult = HttpUtil.postJson(hostUrl + LOGIN_BEFORE_XJU_LOGIC_API, JacksonUtil.parseJson(employeeNo), signature, timestamp);
//        String schoolCodeParam = null;
        if (!StringUtils.isBlank(callResult)) {
            log.info("callbackResult:{}", JacksonUtil.parseJson(callResult));
            Result result = JSON.parseObject(callResult, Result.class);
            if (result.getCode() == HttpStatus.OK.value()) {
                Object data = result.getData();
                List<Map> sysuserList = JSONObject.parseArray(JSON.toJSONString(data), Map.class);
                if (CollectionUtils.isEmpty(sysuserList)) {
                    throw ExceptionResultEnum.ERROR.exception("未查到此用户");
                }
            } else {
                throw ExceptionResultEnum.ERROR.exception("调用知学知考西交大登录前查找账号接口失败");
            }
        }

        Map<String, Object> accessTokenParams = new LinkedHashMap<>();
        accessTokenParams.put("code", code);
        String accessTokenResult = HttpUtil.post(ACCESS_TOKEN_URL, accessTokenParams, null);
        String accessToken = null, gsessionId = null;

        //获取accessToken
        if (Objects.nonNull(accessTokenResult)) {
            log.info("accessTokenResult:{}", JacksonUtil.parseJson(accessTokenResult));
            JSONObject jsonObject = JSONObject.parseObject(accessTokenResult);
            JSONObject object = jsonObject.getJSONObject("data");
            String message = jsonObject.getString("message");
            if (Objects.nonNull(object) && Objects.nonNull(message) && Objects.equals(message, "成功")) {
                accessToken = object.getString("accessToken");
                gsessionId = object.getString("gsessionId");
            } else {
                throw ExceptionResultEnum.ERROR.exception(message);
            }
        }

        OpenParams openParams = null;
        //获取用户信息
        if (Objects.nonNull(accessToken)) {
            String userInfoResult = HttpUtil.post(USER_INFO_URL, null, accessToken);
            if (Objects.nonNull(userInfoResult)) {
                log.info("userInfoResult:{}", JacksonUtil.parseJson(userInfoResult));
                openParams = new OpenParams();
//                openParams.setResult(JacksonUtil.parseJson(userInfoResult));
                JSONObject jsonObject = JSONObject.parseObject(userInfoResult);
                JSONObject object = jsonObject.getJSONObject("data");
                String message = jsonObject.getString("message");
                JSONArray userTypeJsonArray = object.getJSONArray("userTypes");
                JSONArray deptInfoJsonArray = object.getJSONArray("deptInfos");
                if (Objects.nonNull(object) && Objects.nonNull(message) && Objects.equals(message, "成功")) {
                    openParams.setOrgName(object.getString("orgName"));
                    if (Objects.nonNull(userTypeJsonArray) && userTypeJsonArray.size() > 0) {
                        JSONObject userTypeJsonObject = userTypeJsonArray.getJSONObject(0);
                        openParams.setName(userTypeJsonObject.getString("memberName"));
                        Integer userTypeRole = userTypeJsonObject.getInteger("userType");
                        if (Objects.nonNull(userTypeRole) && userTypeRole.intValue() == 1) {
                            openParams.setRoleName("学生");
                        } else if (Objects.nonNull(userTypeRole) && userTypeRole.intValue() == 2) {
                            openParams.setRoleName("教职工");
                        }
                    }
                    if (Objects.nonNull(deptInfoJsonArray) && deptInfoJsonArray.size() > 0) {
                        JSONObject deptInfoJsonArrayJsonObject = deptInfoJsonArray.getJSONObject(0);
                        openParams.setDeptName(deptInfoJsonArrayJsonObject.getString("deptName"));
                    }
                } else {
                    throw ExceptionResultEnum.ERROR.exception(message);
                }
            }
        }

        //登出
        if (Objects.nonNull(gsessionId)) {
            StringJoiner stringJoiner = new StringJoiner("");
            stringJoiner.add(logoutUrl).add(SystemConstant.GET_UNKNOWN).add("gSessionId")
                    .add(SystemConstant.GET_EQUAL).add(gsessionId);
            returnUrl = stringJoiner.toString();
        } else {
            throw ExceptionResultEnum.ERROR.exception("gSessionId为空");
        }
        return new RedirectParams(employeeNo, schoolCode, returnUrl, openParams);
    }

    /**
     * 西交大注销逻辑
     *
     * @param schoolCode
     * @param gSessionId
     * @throws IOException
     */
    public void logout(String schoolCode,
                       String gSessionId) throws IOException {
        if (Objects.isNull(logoutUrl) || Objects.equals(logoutUrl, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("鉴权退出地址不存在");
        }
        if (Objects.isNull(gSessionId) || Objects.equals(gSessionId, "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("sessionId为空");
        }
        authInfoService.appHasExpired(schoolCode);
        Map<String, Object> logoutParams = new LinkedHashMap<>();
        logoutParams.put("gSessionId", gSessionId);
        String logoutResult = HttpUtil.post(LOGOUT_URL, logoutParams, null);
        if (Objects.nonNull(logoutResult)) {
            log.info("logoutResult:{}", JacksonUtil.parseJson(logoutResult));
            JSONObject jsonObject = JSONObject.parseObject(logoutResult);
            String message = jsonObject.getString("message");
            if (Objects.nonNull(message) && Objects.equals(message, "成功")) {
                HttpServletResponse response = ServletUtil.getResponse();
                response.setHeader("Access-Control-Allow-Origin", SystemConstant.PATH_MATCH);
                response.sendRedirect(returnUrl);
            }
        }
    }
}
